from pprint import pprint


class VaccinationCenter:
    def __init__(self, name: str, timeslot_to_capacity: dict[int, int]):
        self.name = name
        self.timeslot_to_capacity = timeslot_to_capacity
        self.timeslots = list(timeslot_to_capacity.keys())


class Person:
    def __init__(self, name: str, preferred_timeslot_to_centers: dict[str, list[int]]):
        self.name = name
        self.preferred_timeslot_to_centers = preferred_timeslot_to_centers

    def get_preferred_timeslot_to_centers(self, center_name: str):
        return self.preferred_timeslot_to_center.get(center_name, None)


class CenterNode:
    def __init__(self, center: VaccinationCenter, timeslot: int, capacity: int):
        self.center = center
        self.timeslot = timeslot
        self.capacity = capacity
        self.connected_persons = []

    def add_person(self, person: Person):
        self.connected_persons.append(person)

    def get_capacity(self):
        return self.capacity

    def get_timeslot(self):
        return self.timeslot

    def get_center(self):
        return self.center


class PersonNode:
    def __init__(self, person_name: str, preferred_timeslot_to_centers: dict[str, list[int]]):
        self.person_name = person_name
        self.preferred_timeslot_to_centers = preferred_timeslot_to_centers
        self.connected_centers = []

    def add_center_node(self, center_node: CenterNode):
        self.connected_centers.append(center_node)


class BiGraph:
    def __init__(self, vaccination_centers: dict[str, VaccinationCenter], persons: list[Person]):
        self.vaccination_centers = vaccination_centers
        self.persons = persons
        self.person_nodes = {}
        self.center_nodes = {}
        self.buildGraph()
        self.drawGraph()

    def buildGraph(self):
        for person in self.persons:
            person_node = PersonNode(
                person.name, person.preferred_timeslot_to_centers)
            self.person_nodes[person.name] = person_node

        for center in self.vaccination_centers:
            for timeslot in center.timeslots:
                center_node = CenterNode(
                    center, timeslot, center.timeslot_to_capacity[timeslot])
                self.center_nodes[(center.name, timeslot)] = center_node

    def drawGraph(self):
        for _, person_node in self.person_nodes.items():
            for timeslot, centers in person_node.preferred_timeslot_to_centers.items():
                for center_name in centers:
                    if((center_name, timeslot) not in self.center_nodes):
                        continue
                    center_node = self.center_nodes[(center_name, timeslot)]
                    person_node.add_center_node(center_node)
                    center_node.add_person(person_node)

    def get_node(self):
        result = {}
        for name, person_nodes in self.person_nodes.items():
            result[name] = person_nodes.connected_centers
        return result

    def find_min_matched(self, all_person_node, matched, unmatched):
        min_person = None
        for person in sorted(all_person_node.values(), key = lambda x: x.person_name):
            # print(person.person_name, list(map(lambda x: f"{x.center.name}, {x.timeslot}", person.connected_centers)))
            if(person in matched or person in unmatched):
                continue
            if(min_person is None or len(person.connected_centers) < len(min_person.connected_centers)):
                min_person = person
        return min_person


    def solve(self, person_name: str = None, inclusive:bool = False):
        matched = {}
        unmatched = []
        all_person_node = self.person_nodes.copy()
        while (len(matched) + len(unmatched) < len(self.person_nodes)):
            curr = self.find_min_matched(all_person_node, matched, unmatched)
            if(inclusive and curr.person_name == person_name):
                unmatched.append(curr)
                return matched, unmatched
            max_node = None
            for center_node in sorted(sorted(curr.connected_centers, key=lambda x: x.center.name), key = lambda x: x.timeslot):
                if(center_node.capacity > 0 and (max_node is None or center_node.capacity > max_node.capacity)):
                    max_node = center_node
            del all_person_node[curr.person_name]
            # print(list(map(lambda x: x.person_name, all_person_node.values())))
            if (max_node is None):
                print(curr.person_name, "is unmatched")
                unmatched.append(curr)
                continue
            print(curr.person_name, "is matched to", max_node.center.name, "at", max_node.timeslot)
            max_node.capacity -= 1
            matched[curr.person_name] = max_node

            if(curr in max_node.connected_persons):
                max_node.connected_persons.remove(curr)
            if(max_node.capacity == 0):
                for person in max_node.connected_persons:
                    person.connected_centers.remove(max_node)
                    person.preferred_timeslot_to_centers[max_node.timeslot].remove(max_node.center.name)
                max_node.connected_persons = []
                
            if curr.person_name == person_name:
                return matched, unmatched
        for center in self.center_nodes.values():
            if(center.capacity != 0):
                print(center.center.name, "at", center.timeslot, "has", center.capacity, "people left")

                
            
        return matched, unmatched
                

