import matplotlib.pyplot as plt
import networkx as nx
from data import vaccination_centers, persons
from BiGraph import BiGraph


def e_set(parition: int, lst: list):
    result = []
    for i, n in enumerate(lst):
        result.append((n, (parition, i)))
    return result


def plot_full_graph():
    # Create a graph
    G = nx.Graph()
    people_s = set()
    center_s = set()

    graph = BiGraph(vaccination_centers, persons)
    for person in persons:
        people_s.add(person.name)
    for center in vaccination_centers:
        for timeslot, capacity in center.timeslot_to_capacity.items():
            center_s.add(f"{center.name}:{timeslot}:{capacity}")

    G.add_nodes_from(people_s)
    G.add_nodes_from(center_s)

    for person_name, centers in graph.get_node().items():
        for center in centers:
            G.add_edge(
                person_name, f"{center.center.name}:{center.timeslot}:{center.capacity}")

    pos = {}
    print(len(persons), len(people_s))
    pos.update(e_set(1, people_s))  # put nodes from X at x=1
    pos.update(e_set(2, center_s))  # put nodes from Y at x=2
    nx.draw(G, pos=pos,  with_labels=True, node_size=1000, font_size=10)
    plt.show()


def plot_person(name: list[str]):
    # Create a graph
    G = nx.Graph()
    people_s = set()
    center_s = set()

    graph = BiGraph(vaccination_centers, persons)
    for person in persons:
        if(person.name not in name):
            continue
        people_s.add(person.name)

    G.add_nodes_from(people_s)
    G.add_nodes_from(center_s)

    for person_name, centers in graph.get_node().items():
        if (person_name not in name):
            continue
        for center in centers:
            center_s.add(
                f"{center.center.name}:{center.timeslot}:{center.capacity}")
            G.add_edge(
                person_name, f"{center.center.name}:{center.timeslot}:{center.capacity}")

    pos = {}
    pos.update(e_set(1, people_s))  # put nodes from X at x=1
    pos.update(e_set(2, center_s))  # put nodes from Y at x=2
    nx.draw(G, pos=pos,  with_labels=True, node_size=1000, font_size=10)
    plt.show()


def solve(names: list = list(map(lambda x: x.name, persons)), draw_person: str = None, inclusive: bool = False):
    graph = BiGraph(vaccination_centers, persons)
    matched, unmatched = graph.solve(draw_person, inclusive)

    G = nx.Graph()
    people_s = set()
    center_s = set()

    for person in persons:
        if(person.name not in names):
            continue
        people_s.add(person.name)

    G.add_nodes_from(people_s)
    G.add_nodes_from(center_s)

    center_map = {}

    for person_name, center in matched.items():
        if(person_name not in names):
            continue
        center_s.add(
            f"{center.center.name}:{center.timeslot}:{center.capacity}")
        center_map[(center.center.name, center.timeslot)] = center
        G.add_edge(
            person_name, f"{center.center.name}:{center.timeslot}:{center.capacity}", color='r')

    for person_node in unmatched:
        if(person_node.person_name not in names):
            continue
        for center in graph.get_node()[person_node.person_name]:
            new_center = center_map.get(
                (center.center.name, center.timeslot), center)
            center_s.add(
                f"{new_center.center.name}:{new_center.timeslot}:{new_center.capacity}")
            G.add_edge(person_node.person_name,
                       f"{new_center.center.name}:{new_center.timeslot}:{new_center.capacity}", color='black')

    for center in vaccination_centers:
        for timeslot in center.timeslots:
            if (center, timeslot) in center_map:
                continue
            center_s.add(
                f"{center.name}:{timeslot}:{center.timeslot_to_capacity[timeslot]}")

    pos = nx.circular_layout(G)
    edges = G.edges()
    colors = [G[u][v]['color'] for u, v in edges]
    pos.update(e_set(1, people_s))
    pos.update(e_set(2, center_s))

    nx.draw(G, pos=pos,  with_labels=True, node_size=1000,
            font_size=10, edge_color=colors)
    plt.show()


if __name__ == "__main__":
    ls = list(map(lambda x: x.name, persons))
    ls.remove('Jerry')
    ls.remove('David')
    ls.remove('Jones')
    # solve(ls)
    plot_full_graph()
    pass
